[supervisord]
nodaemon=true
logfile=/dev/null
logfile_maxbytes=0


[program:text-generation-launcher]
command=text-generation-launcher --max-concurrent-requests=128 --hostname=0.0.0.0 --port=8081 {{extra_args}}
startsecs=0
startretries=1
autostart=true
autorestart=true
stdout_logfile=/dev/fd/1
stdout_logfile_maxbytes=0
redirect_stderr=true

[program:nginx]
command=nginx -g "daemon off;"
startsecs=0
startretries=1
autostart=true
autorestart=true
stdout_logfile=/dev/fd/1
stdout_logfile_maxbytes=0
redirect_stderr=true
